"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Runner = void 0;
var asTable = require("as-table");
var chalk = require("chalk");
var fs = require("fs");
var mkdirp = require("mkdirp");
var moment = require("moment");
var path = require("path");
var interfaces_1 = require("./interfaces");
var Runner = /** @class */ (function () {
    function Runner(envStatus, argv, spinner) {
        this.envStatus = envStatus;
        this.argv = argv;
        this.spinner = spinner;
    }
    Runner.prototype.getArgv = function () {
        return this.argv;
    };
    Runner.prototype.run = function () {
        var _this = this;
        var argv = this.getArgv();
        var config = this.envStatus.getConfig();
        var requestEnv = argv._[0];
        if (argv.init) {
            if (config) {
                console.log(Runner.MESSAGES.CONFIG_ALLREADY_EXIST);
                return Promise.resolve(Runner.MESSAGES.CONFIG_ALLREADY_EXIST);
            }
            else {
                var configPath = path.resolve(__dirname, '../.envstatus.js');
                fs.writeFileSync(path.resolve('.envstatus.js'), fs.readFileSync(configPath));
                console.log(Runner.MESSAGES.CONFIG_FILE_CREATED);
                return Promise.resolve(Runner.MESSAGES.CONFIG_FILE_CREATED);
            }
        }
        if (argv.gen) {
            var data = this.envStatus.getLastCommit(new Date());
            data.project = path.basename(process.cwd());
            var outputPath = path.resolve((config && config.gen) || 'dist/env-status.json');
            mkdirp.sync(path.dirname(outputPath));
            fs.writeFileSync(outputPath, JSON.stringify(data, null, 2));
            console.log(Runner.MESSAGES.COMMIT_LOG_GENERATED);
            return Promise.resolve(Runner.MESSAGES.COMMIT_LOG_GENERATED);
        }
        var spinner = this.spinner;
        spinner.start(Runner.MESSAGES.SPINNER_START);
        if (!config) {
            spinner.fail(Runner.MESSAGES.SPINNER_FAIL_NO_CONFIG);
            return Promise.resolve(Runner.MESSAGES.SPINNER_FAIL_NO_CONFIG);
        }
        var envs = this.getEnvListForPrint(config.envs, requestEnv);
        if (requestEnv && envs.length < 2) {
            spinner.fail(Runner.MESSAGES.SPINNER_FAIL_REQUESTED_ENV_UNDEFINED);
            return Promise.resolve(Runner.MESSAGES.SPINNER_FAIL_REQUESTED_ENV_UNDEFINED);
        }
        spinner.text = Runner.MESSAGES.SPINNER_LOADING_ENV_DATA;
        return Promise.all(envs.map(function (env) { return _this.envStatus.fetchEnvData(env); })).then(function (envsData) { return __awaiter(_this, void 0, void 0, function () {
            var envsData2;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        envsData = envsData.sort(function (a, b) {
                            return _this.getEnvWeight(a.env) - _this.getEnvWeight(b.env) + (a.date > b.date ? -1 : a.date < b.date ? 1 : 0);
                        });
                        return [4 /*yield*/, Promise.all(envsData.map(function (data) { return __awaiter(_this, void 0, void 0, function () {
                                var status, res;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            if (interfaces_1.isEnvErrDataType(data)) {
                                                status = chalk.red(data.err);
                                                return [2 /*return*/, {
                                                        env: data.env,
                                                        status: status,
                                                    }];
                                            }
                                            if (!(data.env === 'production')) return [3 /*break*/, 1];
                                            status = '';
                                            return [3 /*break*/, 3];
                                        case 1: return [4 /*yield*/, this.envStatus.isEnvAvailable(data.env)];
                                        case 2:
                                            if (_a.sent()) {
                                                status = chalk.green('Available');
                                            }
                                            else {
                                                status = chalk.yellow('Using');
                                            }
                                            _a.label = 3;
                                        case 3:
                                            res = {
                                                env: data.env,
                                                status: status,
                                                branch: data.branch,
                                                commit: data.commit,
                                                author: data.author,
                                                date: data.date && moment(data.date).format('MM/DD HH:mm:ss'),
                                                since: data.date && moment(data.date).fromNow(),
                                            };
                                            return [2 /*return*/, res];
                                    }
                                });
                            }); }))];
                    case 1:
                        envsData2 = _a.sent();
                        spinner.stop();
                        this.printTable(envsData2);
                        return [2 /*return*/, Promise.resolve('')];
                }
            });
        }); });
    };
    Runner.prototype.printTable = function (envsData) {
        console.log('');
        console.log(asTable.configure({ delimiter: ' | ' })(envsData));
        console.log('');
    };
    Runner.prototype.getEnvWeight = function (env) {
        if (env === 'production') {
            return 10;
        }
        else if (env === 'staging') {
            return 20;
        }
        else {
            return 30;
        }
    };
    Runner.prototype.getEnvListForPrint = function (envs, requestEnv) {
        return envs.filter(function (env) { return (requestEnv ? env === requestEnv || env === 'production' : true); });
    };
    Runner.MESSAGES = {
        CONFIG_ALLREADY_EXIST: chalk.yellow('.envstatus.js file already exists!'),
        CONFIG_FILE_CREATED: chalk.green('.envstatus.js file created!'),
        COMMIT_LOG_GENERATED: chalk.green('Commit log generated!'),
        SPINNER_START: 'Loading .envstatus.js',
        SPINNER_FAIL_NO_CONFIG: chalk.yellow('.envstatus.js') + " file is missing!",
        SPINNER_FAIL_REQUESTED_ENV_UNDEFINED: "Requested env undefined!",
        SPINNER_LOADING_ENV_DATA: 'Loading envs data',
    };
    return Runner;
}());
exports.Runner = Runner;
