"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Runner = void 0;
var chalk = require("chalk");
var Runner = /** @class */ (function () {
    function Runner(envStatus, argv) {
        this.envStatus = envStatus;
        this.argv = argv;
    }
    Runner.prototype.getArgv = function () {
        return this.argv;
    };
    Runner.prototype.run = function () {
        var _this = this;
        var argv = this.getArgv();
        var targetBranchName = argv._[1] || '';
        var branchName = argv._[0] || '';
        if (!branchName || !targetBranchName) {
            console.log(chalk.red('Please specify source branch and target branch.'));
            return Promise.resolve(20);
        }
        var targetBranchType = this.envStatus.getBranchType(targetBranchName);
        var branchType = this.envStatus.getBranchType(branchName);
        return this.envStatus
            .fetchOrigin()
            .then(function () {
            if ((branchType === "SPRINT" /* SPRINT */ || branchType === "HOTFIX" /* HOTFIX */) &&
                targetBranchType !== "MASTER" /* MASTER */) {
                console.log(chalk.red('Sprint and hotfix branch must be merged into master branch.'));
                return 1;
            }
            if ((branchType === "SPRINT_FEATURE" /* SPRINT_FEATURE */ || branchType === "SPRINT_FIX" /* SPRINT_FIX */) &&
                targetBranchType !== "SPRINT" /* SPRINT */) {
                console.log(chalk.red('Feature and fix branch must be merged into sprint branch.'));
                return 2;
            }
            if (branchType === "MASTER" /* MASTER */ || branchType === "OTHERS" /* OTHERS */) {
                console.log(chalk.red("Source branch name \"" + branchName + "\" is invalid."));
                return 3;
            }
            var branchCommit;
            var targetBranchCommit;
            try {
                branchCommit = _this.envStatus.getBranchLastCommitId('origin/' + branchName);
            }
            catch (err) {
                console.log(chalk.red("Failed to get last commit of \"" + branchName + "\"."));
                return 4;
            }
            try {
                targetBranchCommit = _this.envStatus.getBranchLastCommitId('origin/' + targetBranchName);
            }
            catch (err) {
                console.log(chalk.red("Failed to get last commit of \"" + targetBranchName + "\"."));
                return 5;
            }
            if (!_this.envStatus.isAncestorCommit(targetBranchCommit, branchCommit)) {
                console.log(chalk.red("Please merge lastest commits from \"" + targetBranchName + "\" into \"" + branchName + "\" first."));
                return 6;
            }
            return 0;
        })
            .catch(function () {
            console.log(chalk.red('Failed to fetch origin.'));
            return 10;
        });
    };
    return Runner;
}());
exports.Runner = Runner;
